﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace PlanetTutorial
{
    public enum PlanetType
    {
        Ice,
        Water,
        Rock,
        Gas,
        Molten,
        Undefined
    };
    public abstract class Planet
    {
        public int Seed;
        public PlanetType Type;
        public Texture2D PermTex;
        public Texture2D Mercator;
        public Texture2D Palette;
        public Texture2D normalmap;
        public Texture2D lava;
        public Effect terrain;
        public Effect draw;
        public Effect atmosphere;
        
        public RenderTargetState rts;
        public double Nz;
        public float xOvercast = 1.1f;
        public int TextureWidth=512;
        public int TextureHeight=512;
        public int SubType;
        public float roll;
        public float pitch;

        float p_radius = 200;
        float a_radius = 205;
        float c_radius = 202;

        GraphicsDevice graphics;

        public void Draw(Matrix View, Matrix Projection)
        {
            Matrix World = Matrix.CreateScale(p_radius) * Matrix.CreateRotationY(roll) * Matrix.CreateRotationX(pitch);
            Matrix wvp =  World * View * Projection;
            Vector3 light = -Game1.LightPosition;
            light.Normalize();
            draw.Parameters["LightDirection"].SetValue(light);
            draw.Parameters["wvp"].SetValue(wvp);
            draw.Parameters["Palette"].SetValue(Palette);
            draw.Parameters["ColorMap"].SetValue(Mercator);
            draw.Parameters["BumpMap"].SetValue(normalmap);
            draw.Parameters["world"].SetValue(World);
            draw.Parameters["subtype"].SetValue(SubType / 8.0f);
            if (draw.Parameters["Lava"]!=null)
                draw.Parameters["Lava"].SetValue(lava);

           
            
            for (int pass = 0; pass < draw.CurrentTechnique.Passes.Count; pass++)
            {
                for (int msh = 0; msh < Game1.sphere.Meshes.Count; msh++)
                {
                    ModelMesh mesh = Game1.sphere.Meshes[msh];
                    
                    for (int prt = 0; prt < mesh.MeshParts.Count; prt++)
                        mesh.MeshParts[prt].Effect = draw;
                    mesh.Draw();
                }
            }

            if (Type == PlanetType.Water)
            {
                Game1.cloudgen.DrawClouds(View, Matrix.CreateScale(c_radius) * Matrix.CreateRotationY(roll) * Matrix.CreateRotationX(pitch), Projection, SubType);
            }
            if (Type == PlanetType.Molten)
            {
                Game1.cloudgen.DrawClouds(View, Matrix.CreateScale(c_radius) * Matrix.CreateRotationY(roll) * Matrix.CreateRotationX(pitch), Projection, 2);
            }
            if (atmosphere != null)
            {
                float scale = 1.0f / (a_radius - p_radius);

                atmosphere.Parameters["fOuterRadius"].SetValue(a_radius);
                atmosphere.Parameters["fInnerRadius"].SetValue(p_radius);
                atmosphere.Parameters["fOuterRadius2"].SetValue(a_radius * a_radius);
                atmosphere.Parameters["fInnerRadius2"].SetValue(p_radius * p_radius);
                atmosphere.Parameters["fKr4PI"].SetValue(0.0025f * 4 * MathHelper.Pi);
                atmosphere.Parameters["fKm4PI"].SetValue(0.0015f * 4 * MathHelper.Pi);
                atmosphere.Parameters["fScale"].SetValue(scale);
                atmosphere.Parameters["fScaleDepth"].SetValue(0.25f);
                atmosphere.Parameters["fScaleOverScaleDepth"].SetValue(scale / 0.25f);
                atmosphere.Parameters["fSamples"].SetValue(2.0f);
                atmosphere.Parameters["nSamples"].SetValue(2);
                switch (Type)
                {
                    case PlanetType.Gas:
                        atmosphere.Parameters["fKrESun"].SetValue(0.00025f * 5);
                        atmosphere.Parameters["fKmESun"].SetValue(0.0015f * 5);
                        atmosphere.Parameters["v3InvWavelength"].SetValue(new Vector3(1.0f / (float)Math.Pow(0.650f, 4), 1.0f / (float)Math.Pow(0.570f, 4), 1.0f / (float)Math.Pow(0.475f, 4)));
                        break;
                    case PlanetType.Molten:
                        atmosphere.Parameters["fKrESun"].SetValue(0.00025f * 5);
                        atmosphere.Parameters["fKmESun"].SetValue(0.0015f * 10);
                        atmosphere.Parameters["v3InvWavelength"].SetValue(new Vector3(1.0f / (float)Math.Pow(0.350f, 4), 1.0f / (float)Math.Pow(0.970f, 4), 1.0f / (float)Math.Pow(0.975f, 4)));
                        break;
                    default:
                        atmosphere.Parameters["fKrESun"].SetValue(0.00025f * 10);
                        atmosphere.Parameters["fKmESun"].SetValue(0.0015f * 10);
                        atmosphere.Parameters["v3InvWavelength"].SetValue(new Vector3(1.0f / (float)Math.Pow(0.650f, 4), 1.0f / (float)Math.Pow(0.570f, 4), 1.0f / (float)Math.Pow(0.475f, 4)));
                        break;
                }

                World = Matrix.CreateScale(a_radius) * Matrix.CreateRotationX(pitch);
                Vector3 vl = - Game1.LightPosition;
                vl.Normalize();

                atmosphere.Parameters["World"].SetValue(World);
                atmosphere.Parameters["View"].SetValue(View);
                atmosphere.Parameters["Projection"].SetValue(Projection);
                atmosphere.Parameters["v3CameraPos"].SetValue(Game1.CameraPosition);

                atmosphere.Parameters["v3LightDir"].SetValue(vl);
                atmosphere.Parameters["v3LightPos"].SetValue(Game1.LightPosition);
                atmosphere.Parameters["fCameraHeight"].SetValue(Game1.CameraPosition.Length());
                atmosphere.Parameters["fCameraHeight2"].SetValue(Game1.CameraPosition.LengthSquared());

                graphics.RenderState.CullMode = CullMode.CullClockwiseFace;
                graphics.RenderState.DepthBufferEnable = false;
                foreach (ModelMesh mesh in Game1.sphere.Meshes)
                {
                    for (int i = 0; i < mesh.MeshParts.Count; i++)
                    {
                        // Set this MeshParts effect (currentEffect) to the desired effect (from .fx file)   
                        mesh.MeshParts[i].Effect = atmosphere;
                    }
                    mesh.Draw();
                }
                graphics.RenderState.CullMode = CullMode.None;
                graphics.RenderState.DepthBufferEnable = true;
            }
        }
        

        public void Generate(GraphicsDevice _graphics)
        {
            graphics = _graphics;
            rts = new RenderTargetState(graphics, TextureWidth, TextureHeight, TextureWidth, TextureHeight);
            rts.BeginRenderToTexture();
            graphics.Clear(new Color(128, 128, 255, 255));
            graphics.RenderState.DepthBufferEnable = false;
            graphics.RenderState.CullMode = CullMode.None;
            terrain.Parameters["ColorMap"].SetValue(PermTex);
            if (terrain.Parameters["xOvercast"] != null)
                terrain.Parameters["xOvercast"].SetValue(xOvercast);
            if (terrain.Parameters["subtype"] != null)
                terrain.Parameters["subtype"].SetValue(SubType / 4.0f);
            foreach (ModelMesh mesh in Game1.sphere.Meshes)
            {
                for (int i = 0; i < mesh.MeshParts.Count; i++)
                {
                    // Set this MeshParts effect (currentEffect) to the desired effect (from .fx file)   
                    mesh.MeshParts[i].Effect = terrain;
                }

                mesh.Draw();
            }
            Mercator = rts.EndRenderGetTexture();
            //Mercator.Save("Mercator.png", ImageFileFormat.Png);
            rts.DestroyBuffers();
            rts = null;

            Color[] Map = new Color[TextureHeight * TextureWidth];
            Mercator.GetData<Color>(Map);

            normalmap = new Texture2D(graphics, TextureWidth, TextureHeight, 1, TextureUsage.None, SurfaceFormat.Color);
            Color[] pixels = new Color[TextureHeight * TextureWidth];
            Color c3;

            #region Fix the crenalations at the top and bottom of the heightmap
            // Fix the heightmap at the poles.
            int max_x = (TextureHeight - 1) * TextureWidth;
            for (int x = 0; x < TextureWidth; x++)
            {
                Map[x + max_x - TextureWidth] = Map[x + max_x - (TextureWidth * 2)];
                Map[x + max_x] = Map[x + max_x - TextureWidth];
                Map[x + TextureWidth] = Map[x + (TextureWidth * 2)];
                Map[x] = Map[x + TextureWidth];

            }
            //max_x -= TextureWidth;
            for (int x=0; x<TextureWidth; x+=16)
            {
                for (int y = 0; y < 32; y++)
                {
                    int i = y / 2;
                    for (int j = i+1; j <= 16; j++)
                    {
                        Map[x + j + (y * TextureWidth)] = Map[x + i + (y * TextureWidth)];
                        int mx = x + j + max_x - (y * TextureWidth);
                        if (mx<TextureWidth*TextureHeight)
                            Map[mx] = Map[x + i + max_x - (y * TextureWidth)];
                    }
                }
            }
            Mercator.SetData<Color>(Map);
            Mercator.Save("mercator.png", ImageFileFormat.Png);
            #endregion

            for (int y = 0; y < TextureHeight; y++)
            {
                int offset = y * TextureWidth;
                for (int x = 0; x < TextureWidth; x++)
                {

                    float h0 = (float)Map[x + (TextureWidth * y)].R;
                    float h1 = (float)Map[x + (TextureWidth * safey(y + 1))].R;
                    float h2 = (float)Map[safex(x + 1) + (TextureWidth * y)].R;

                    float Nx = h0 - h2;
                    float Ny = h0 - h1;

                    Vector3 Normal = new Vector3((float)Nx, (float)Ny, (float)Nz);
                    Normal.Normalize();
                    Normal /= 2;

                    byte cr = (byte)(128 + (255 * Normal.X));
                    byte cg = (byte)(128 + (255 * Normal.Y));
                    byte cb = (byte)(128 + (255 * Normal.Z));
                    c3 = new Color(cr, cg, cb);
                    pixels[x + (y * TextureWidth)] = c3;
                }
            }
            
            normalmap.SetData<Color>(pixels);
        }

        int safex(int x)
        {
            if (x >= TextureWidth)
                return x - TextureWidth;
            if (x < 0)
                return TextureWidth + x;
            return x;
        }

        int safey(int y)
        {
            if (y >= TextureHeight)
                return TextureHeight-1;
            if (y < 0)
                return 0;
            return y;
        }

        public void BuildPerm(GraphicsDevice graf)
        {
            float[] map = new float[256];
            Random r = new Random(Seed);
            Vector4[] perm = new Vector4[256 * 256];
            int index = 0;
            for (int y = 0; y < 256; y++)
            {
                for (int i = 0; i < 256; i++)
                {
                    Vector4 a = new Vector4();
                    a.X = (float)((r.NextDouble()));
                    a.Y = (float)((r.NextDouble()));
                    a.Z = (float)((r.NextDouble()));
                    a.W = 1;
                    a.Normalize();
                    perm[index] = a;
                    index++;
                }
            }
            for (int i = 0; i < 256; i++)
            {
                map[i] = ((float)i) / 256.0f;
            }
            for (int i = 0; i < 256; i++)
            {
                int k = r.Next(255);
                float l = map[k];
                map[k] = map[i];
                map[i] = l;
            }
            for (int i = 0; i < 256; i++)
            {
                perm[i].W = map[i];
            }
            PermTex = new Texture2D(graf, 256, 256, 1, TextureUsage.None, SurfaceFormat.Vector4);
            PermTex.SetData<Vector4>(perm);

            SubType = r.Next(8);
        }

    }
}
